(* Copyright (C) 2012--2015 Microsoft Research and INRIA *)
module PMS

open Platform.Bytes
open TLSConstants
open CoreCrypto

type rsarepr = bytes

(*private*)
type rsaseed = {seed: rsarepr}

// To ideally avoid collisions concerns between Honest and Coerced pms we use a sum type. 
type rsapms =
  | IdealRSAPMS of rsaseed
  | ConcreteRSAPMS of rsarepr

//this function is used to determine whether idealization should be performed
let honestRSAPMS (pk:RSAKey.pk) (cv:TLSConstants.ProtocolVersion) pms =
  match pms with
  | IdealRSAPMS(s) -> true
  | ConcreteRSAPMS(s) -> false

let genRSA (pk:RSAKey.pk) (vc:ProtocolVersion): rsapms =
    let verBytes = versionBytes vc in
    let rnd = CoreCrypto.random 46 in
    let pms = verBytes @| rnd in
        ConcreteRSAPMS(pms)

let coerceRSA (pk:RSAKey.pk) (cv:ProtocolVersion) b = ConcreteRSAPMS(b)

let leakRSA (pk:RSAKey.pk) (cv:ProtocolVersion) pms =
  match pms with
  | ConcreteRSAPMS(b) -> b

// The trusted setup for Diffie-Hellman computations
open CoreCrypto.Keys

type dhrepr = bytes

(*private*) 
type dhseed = {seed: dhrepr}

// To ideally avoid collisions concerns between Honest and Coerced pms we use a sum type.
type dhpms =
  | ConcreteDHPMS of dhrepr

let sampleDH dhp (gx:DHGroup.elt) (gy:DHGroup.elt) =
    let gz = DHGroup.genElement dhp in
    ConcreteDHPMS(gz)

let coerceDH (dhp : dh_params) (gx:DHGroup.elt) (gy:DHGroup.elt) b = ConcreteDHPMS(b)

let coerceECDH (dhe : ec_params) (gx:ECGroup.point) (gy:ECGroup.point) b = ConcreteDHPMS(b)

type pms =
  | RSAPMS of RSAKey.pk * ProtocolVersion * rsapms
  | DHPMS of CommonDH.parameters * CommonDH.element * CommonDH.element * dhpms
